@@ -1,10 +1,8 @@
+use crate::memory::output::OutputResult;
+use jl_sys::jl_value_t;
 use std::ptr::NonNull;
 
-use jl_sys::jl_value_t;
-
-use crate::memory::output::OutputResult;
-
-pub trait OutputResultExt {
+pub(super) trait OutputResultExt {
     fn unwrap_non_null(self) -> NonNull<jl_value_t>;
 }
 
