@@ -2,7 +2,10 @@
 
 use super::frame::Frame;
 use crate::Julia;
-use jl_sys::{jl_gc_collect, jl_gc_collection_t, jl_gc_enable, jl_gc_is_enabled, jl_gc_safepoint};
+use jl_sys::{
+    jl_flush_cstdio, jl_gc_collect, jl_gc_collection_t, jl_gc_enable, jl_gc_is_enabled,
+    jl_gc_safepoint,
+};
 
 /// The different collection modes.
 #[derive(Debug, Copy, Clone)]
@@ -33,6 +36,11 @@ pub trait Gc: private::Gc {
     unsafe fn gc_safepoint(&mut self) {
         jl_gc_safepoint();
     }
+
+    /// Flush C stdio.
+    fn flush_io(&mut self) {
+        unsafe { jl_flush_cstdio() }
+    }
 }
 
 impl Gc for Julia {}
