@@ -39,14 +39,14 @@ impl_valid_layout!(Expr<'scope>, 'scope);
 impl_debug!(Expr<'_>);
 
 impl<'scope> Wrapper<'scope, '_> for Expr<'scope> {
-    type Internal = jl_expr_t;
+    type Wraps = jl_expr_t;
     const NAME: &'static str = "Expr";
 
-    unsafe fn wrap_non_null(inner: NonNull<Self::Internal>, _: Private) -> Self {
+    unsafe fn wrap_non_null(inner: NonNull<Self::Wraps>, _: Private) -> Self {
         Self(inner, PhantomData)
     }
 
-    unsafe fn unwrap_non_null(self, _: Private) -> NonNull<Self::Internal> {
+    fn unwrap_non_null(self, _: Private) -> NonNull<Self::Wraps> {
         self.0
     }
 }
