@@ -30,14 +30,14 @@ impl_debug!(WeakRef<'_>);
 impl_valid_layout!(WeakRef<'scope>, 'scope);
 
 impl<'scope> Wrapper<'scope, '_> for WeakRef<'scope> {
-    type Internal = jl_weakref_t;
+    type Wraps = jl_weakref_t;
     const NAME: &'static str = "WeakRef";
 
-    unsafe fn wrap_non_null(inner: NonNull<Self::Internal>, _: Private) -> Self {
+    unsafe fn wrap_non_null(inner: NonNull<Self::Wraps>, _: Private) -> Self {
         Self(inner, PhantomData)
     }
 
-    unsafe fn unwrap_non_null(self, _: Private) -> NonNull<Self::Internal> {
+    fn unwrap_non_null(self, _: Private) -> NonNull<Self::Wraps> {
         self.0
     }
 }
